package com.sybotan.android.demo.tools;

import android.content.Context;
import android.content.res.TypedArray;
import android.util.AttributeSet;
import android.view.View;
import android.view.ViewGroup;

import com.sybotan.android.demo.R;

/**
 * Created by cunwen on 17/4/14.
 */
public class WordWrapView extends ViewGroup {
    private int PADDING_HOR = 7;//水平方向padding
    private int PADDING_VERTICAL = 5;//垂直方向padding--文字上下距离边框的距离
    private int SIDE_MARGIN = 20;//左右间距--整体的左右margin
    private int TEXT_MARGIN = 6;//textview之间的间距
    private boolean mIsAudit;

    /**
     * @param context
     */
    public WordWrapView(Context context) {
        this(context, null);
    }

    /**
     * @param context
     * @param attrs
     * @param defStyle
     */
    public WordWrapView(Context context, AttributeSet attrs, int defStyle) {
        super(context, attrs, defStyle);
        TypedArray array = context.obtainStyledAttributes(attrs, R.styleable.wrapView);
        PADDING_HOR = DensityUtils.dp2px(context, array.getInt(R.styleable.wrapView_padding_hor, 7));
        PADDING_VERTICAL = DensityUtils.dp2px(context, array.getInt(R.styleable.wrapView_padding_ver, 5));
        SIDE_MARGIN = DensityUtils.dp2px(context, array.getInt(R.styleable.wrapView_side_margin, 0));
        TEXT_MARGIN = DensityUtils.dp2px(context, array.getInt(R.styleable.wrapView_text_margin, 12));
        mIsAudit = array.getBoolean(R.styleable.wrapView_isAudit, false);
        array.recycle();
    }


    /**
     * @param context
     * @param attrs
     */
    public WordWrapView(Context context, AttributeSet attrs) {
        this(context, attrs, 0);
    }


    @Override
    protected void onLayout(boolean changed, int l, int t, int r, int b) {
        int childCount = getChildCount();
        int autualWidth = r - l;
        int x = SIDE_MARGIN;// 横坐标开始
        int y = 0;//纵坐标开始
        int rows = 1;
        for (int i = 0; i < childCount; i++) {
            View view = getChildAt(i);
//            view.setBackgroundColor(Color.WHITE);
            int width = view.getMeasuredWidth();
            int height = view.getMeasuredHeight();

            x += width + TEXT_MARGIN;
            if (rows == 1) {
                if (x - TEXT_MARGIN > autualWidth) {
                    x = width + SIDE_MARGIN;
                    rows++;
                }
            } else {
                if (x > autualWidth) {
                    x = width + SIDE_MARGIN;
                    rows++;
                }
            }

            y = rows * (height + TEXT_MARGIN);
            if (rows == 1) {
                view.layout(x - width - TEXT_MARGIN, y - height, x - TEXT_MARGIN, y);
            } else {
                view.layout(x - width, y - height, x, y);
            }
        }
    }

    ;

    @Override
    protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {
        int x = 0;//横坐标
        int y = 0;//纵坐标
        int rows = 1;//总行数
        int specWidth = MeasureSpec.getSize(widthMeasureSpec);
        int actualWidth = specWidth - SIDE_MARGIN * 2;//实际宽度
        int childCount = getChildCount();
        for (int index = 0; index < childCount; index++) {
            View child = getChildAt(index);
            child.setPadding(PADDING_HOR, PADDING_VERTICAL, PADDING_HOR, PADDING_VERTICAL);
            child.measure(MeasureSpec.UNSPECIFIED, MeasureSpec.UNSPECIFIED);
            int width = child.getMeasuredWidth();
            int height = child.getMeasuredHeight();
            x += width + TEXT_MARGIN;
            if (!mIsAudit) {
                if (index != childCount - 1 && x > actualWidth) {//换行
                    x = width;
                    rows++;
                }
            } else {
                if (x > actualWidth) {//换行
                    x = width;
                    rows++;
                }
            }
            y = rows * (height + TEXT_MARGIN);
        }
        setMeasuredDimension(actualWidth, y);
    }

}
